/*
This program is free software: you can redistribute it and/or modify
 it under the terms of the Attribution 4.0 International as published by
 Creative Commons.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 
 You should have received a copy of the  Attribution 4.0 International as published by
 Creative Commons or visit http://creativecommons.org/licenses/by/4.0/legalcode
 
 Original Author: Jeffrey Nygaard
 Company: Northern Lights Electronic Design, LLC
 Contact: JNygaard@NLEDShop.com
 Date Updated: October 28, 2015
 Webpage: www.NLEDShop.com/nledmatrix
 Written in Processing v3.0 - www.Processing.org
 Software Version: 2b
 
 3rd Party Libraries Utilized:
 netP5 - http://www.sojamo.de/libraries/oscP5/
 Minim - http://code.compartmental.net/tools/minim/
 TheMIDIbus - http://www.smallbutdigital.com/themidibus.php
 DirectShow - http://www.humatic.de/htools/dsj.htm
 
 This software is offered for free and is open-source. You may use it for your commercial projects as described in the license. You may modify
 it to suit you, and share your code if you wish, as long as you give attribution and release it under the same license. It is encouraged to submit 
 improvements and bug fixes so they can be added to the main release version.
 
 Compatible High Quality, LED Controllers for WS2801, WS2803, WS2811, WS2812, WS2812B, LDP8806, along with standard LEDs, and LED Strip  
 are available for purchase from the store. In addition to data interface controllers, a selection of stand-alone with powerful fimrware will soon be
 available.
 
 Please utilize and enjoy the software however you like. Please support the development and improvements, a quite a bit of time has been spent
 producing this software for all to use and modify as they like. If you are able to, please donate or purchase a controller from the store.
 
 I am always interested in developing new devices and software, helping people on their projects, and maybe doing some collabs
 Please contact me at the address above. This is the initial release, and is still pretty rough and documentation isn't completed. Please contact 
 Jnygaard@NLEDshop.com with any questions, bug reports, feature requests, critiques, anything at all, I want to hear about it.
 
 //============================================================================================================
 
 Intro:
 Originally this was going to be a very simple program that converted a movie file into serial data and send it out using a custom patch. But it grew and
 grew into the current behemoth, over time I have added nearly every feature I can think of, or thought was reasonable enough to support. I want
 this software to be available to everyone, to use how they like. Building custom hardware interfaces should be easy enough and is encouraged.
 But please if posting projects online or to social media, please give a link back to the original software. And if you find it very useful you can 
 donate to future development or purchase a controller from Northern Lights Electronic Design.
 
 Description:
 NLED Matrix is a Java based application used for mixing and controlling video content and to output it to a custom DIY LED Matrix or matrices, which can
 be of any shape and size. NLED-Matrix supports pixel patching, which allows the user to create the order in which the software outputs the data to the LED
 Pixels. That means any shape of LED matrix, any layout, any color order, with any type of control scheme can be made to work with the software. It allows
 two feeds of video data, either from a file such as .MOV, from an image, from a external video feed, or from any of the numerous generated content. The 
 mixed video feed can be sent over Serial(USB Adapter, or RS-485 adapter), TCP, or UDP.  
 
 Video feed sources can be video files(AVI, MPG, MOV, and others), one of the several generated content(Meta balls, Spiral, star field, plasma, 
 sound reactive sine, sound reactive bars), custom .dat  files(text files containing the video data, videos converted using a different Processing
 program), DShow input(camera feed, such as screen grabber on Milkdrop or other visualizer)
 
 More generated feeds will be written and added in the future. If you have any you would like to get added in to NLED-Matrix, contact the author.
 
 There are a few options available to transmit the data, such as the basic Serial, to connect to something such as an USB to Serial Adapter, or a
 USB Enabled Microcontroller. Or even directly to most Arduino boards. And it supports TCP or UDP communication of a network or internet. Tested it
 with TCP and UDP connections to an X-Bee Wifi, worked well enough.
 
 //============================================================================================================
 
 DShow/Web Cam Input/screen capture notes
	- use Many Cam, window mode. Screen caputre Milkdrop
		- milkdrop wouldn't stop freezing, tried to caputre full window, didn't work, reverted to window and it worked just fine after that....
                - tried again and confrim that milkdrop would freeze if many cam was running, did options set to full screen
	- zoom in with many cam and turn off the logo watermark
	- For MAC and Linux users, you will have to comment out/delete all the Direct Show code, its all labled
 
 
 Good Mildrop 2 Presets for low res displays:
	Light in the sky, martin-deep blue, martin-glass corridor, Orb-Acid Cycle, Orb-Acid Cycle Gas Giant, Orb-Acid Lamp,Orb-Dark Omen,
		redi jedi-i don't think, long name - machine code, psychotic flower gelatine burst, BrainStain-Blackwidow, Eos+Phat mergent factors-krash's beat detection
		flexi-predator prey spirals, Geiss-Cosmic Dust 2, Goody-Need, Goody-Need Desire Remix, MStress & Zylot - Acid UFO
		flexi-opeartion fatcap II, EOS & Geiss glowsticks v2 03 music shifter edit
 
 Revisions 2a to 2b
 
   - had accidently broke intensity control trying to fix something else, fixed now.
  
 
 
 Revisions 1a to 2a
	 
	 - Ported to processing 3.0
		- open() changed to launch()
		- frame.setresizable() to surface.setResizable()
		- size() to surface.setSize()
	 
	- new config file, any version 1a files will have to  be ported to 2a format
		- dropped COM2 config, added NLED Aurora CMD option

	- Generation functions are now classes, so multiple of the same one can be ran at the same time

	- Quickfeed IDs have been altered, quickfeed files will need updating
	 
	- Made generated feeds into objects so two can be ran at the same time, also easier to follow

	- Moved all matrix configurations to "Matrix" object rather than numerous variables

	- Adjusted how the window is resized due to changes within processing and the generated feeds objects
	 
	- Reset Buttons(A&B) now properly reset the speed, color effects, contrast, invert etc. 
	 
	- Changed how intensity(via the 2 sliders, with lock button) is displayed. No longer is the previews affected by it, but the output data will be.
		Also added percentage values to the lower corners of A & B previews.
	 
	- Changed all the generated method ID numbers, updated the javascript/html generator 
	 
  
Known Issues:

	- not loading slider text values correctly, but still work - which ones?

	- DAT file not compatible with more than 2 feeds, also it freezes
		- need to try it in native resolution see if it still freezes, seems to run several times first though

	
 OLD NOTES:

	 Automatic Mode Works but needs work,
	 Per Line: Fader starting Position - Feed ID - QuickFeed Variable - Draw Counts - Fade Speed(1000 instant, 1+ = increment on draw)
	 
	 Load Files Fill Variables -> set Fader Position -> set FeedID to QuickFeedVar -> Load it ->Set AutoSequenceRunning -> Start decf DrawCounter -> when reaches 0 ->
	 Start Auto Blend -> every Draw add 0.001 for every 1(out of 1000) of speed eg: Fade Speed is 50, add 0.05 -> once it reaches 1 or 0(depending on FeedID)
	 -> Start Over
	 
 
Future Upgrades:
	 
	 - add animation for images, scrolling?? sliding
		- how???? Gen options, random X, random Y, Speed, ???	 
	 
	 - RGBW pixels will not work with current code since it frequently relies on color() and pixel[] variables, not sure how to proceed on that, along
	 with not having access to a RGBW pixel array for testing. But framework is in for the most part, I made most functions compatible for the future
	 
	 - Each feed can be two feeds mixed already, so say a mask layer and color layer
	 
	 - Automatic Mode, it is bare bones right now, needs some work. But is functioning.
	 
	 - Be able to change Auto Fade speed - now a defined constant 
	 
	 - Ensure letter cases are stripped when loading and comparing filenames, hate being that specific
	 
	 - SendData() - Nonstandard RGB data order other than GRB aren't written, but would be simple enough
	 
	 - Add frame rate slider or textfield to menu, so can be changed on the fly
	 
	 - With Graphics Buffers  various blendModes() and filter() could be used on them
	 
 */
 
 
import processing.video.*; //as of version 2a, requires library version 1(not 1.01)
import processing.serial.*;

import javax.swing.ImageIcon;

import themidibus.*; 
//Message from TheMIDIBus Author: Mac users may want to install mmj, a Mac OS X universal binary java MIDI subsystem - or an equivalent third party MIDI subsystem - because the apple implementation 
//	is lacking support for a number of advanced MIDI features. Mmj is recommended because it works, it's free and it's almost open source (CC licence). Please check README.md for more information.
//	If you do use MMJ you will want to set MidiBus.sendTimestamps(boolean sendTimestamps) to false, otherwise messages won't be sent.


//import ddf.minim.analysis.*;
import ddf.minim.*; //for sound reactivity, opens line-in. Should work on all OS

//this will work on windows only Mac and Linux, comment out these two libraries and find the 4 other code sections to comment out
import de.humatic.dsj.*; //for DShow feed input, 
import java.awt.image.BufferedImage; //used for DShow feed input

//loads network communication libraries
import processing.net.*; 	//used for UDP communication
import netP5.*;	        //used for TCP communication

//================================= END IMPORTS ==================================

final color ColButtonsBG = color(60, 60, 60);
final color ColButtonsHL = color(170, 0, 0);
final color ColButtonsOut = color(0, 100, 255); //selected/highlighted color for buttons
final color cFocusedColor = color(0, 0, 255); //focus color for quickfeed button
final color cBackgroundColor = color(200); //background of the software

final color cSliderHLColor = color(100, 100, 100);
final color cSliderBGColor = color(255);
final color cBlendButtStroke = color(145, 40, 150);
final color cSliderHndlColor = color(140, 20, 20);
final color cButtonTextColor = color(255);

final color cAutomaticStroke = color(45, 50, 145);
final color cAutomaticHL = color(0, 100, 50);

final color cMenuStrokeColor = color(0, 0, 255);
final color cMenuColor = color(160); //background color of the menus
final color cMenuButtonsStroke = color(0, 100, 50);
final color cMenuBGColor = color(200);
final color cMenuFontColor = color(0);
final color cSliderStrokeColor = color(0);

final int cDefaultSpeed = 110;
final int cDefaultSpeedX1 = 110;
final int cMaxQuickFeeds = 60;
final int cDefaultAutoFadeSpeed = 100;


final String[] cMetaBallsLabels = {
  "Ball Amt", "Blob Size", "Frequency", "Color Mode"
};
final String[] cSpiralLabels = {
  "Coils", "Offset Color", "ColorMode", "Trails"
};
final String[] cStarFieldLabels = {
  "Shape", "Star Size", "Fill/Stroke", "Amount"
};
final String[] cRipplesLabels = {
  "Shape", "Weight", "Not Used", "Not Used"
};
final String[] cSpinningSquare = {
  "Fill Color", "Stroke Color", "Stroke Weight", "Base Size"
};
final String[] cTextLabels = {
  "Text Size", "Speed", "Y Position", "Modifier"
};
final String[] cFallingBlocks = {
  "Size", "Spacing", "Decay", "Direction"
};
final String[] cSineWave = {
  "Amplitute", "Decay", "Width", "None"
};

final String[] cReactiveBars = {
  "Amplitute", "Width", "Decay", "Spacing"
};

final String[] cNotUsedLabels = {
  "Not Used", "Not Used", "Not Used", "Not Used"
};

final int cTempGUI2X = 150;
final int cTempGUI2Y = 250;

//==================== END CONSTANTS/FINALS == START VARIABLES ================================================

//System Variables
int[] PatchCoordX = new int[1];
int[] PatchCoordY = new int[1];
int[] BufferA = new int[1];
int[] BufferB = new int[1];
int[] MixedBuffer = new int[1];

int[] PrevBufferA = new int[1];
int[] PrevBufferB = new int[1];


byte[] TransmissionArray = new byte[1]; //resized when the device patch file is loaded

int[] ReceivedSerialControl = new int[12];
int[] PrevReceivedSerialControl = new int[12];

//holds .DAT data bytes
byte[] ByteBufferA = new byte[1]; //resized based on Matrix Size
byte[] ByteBufferB = new byte[1]; //resized based on Matrix Size
int DataFrameFeedA = 0; //for .dat file frame counter
int DataFrameFeedB = 0; //for .dat file frame counter


ArrayList strs = new ArrayList(); //used for file saving and loading

//User Configs
int MatrixWidth = 22; //Set in Config file
int MatrixHeight = 22; //Set in Config file
int MaxChannels = (MatrixWidth * MatrixHeight) * 3; //calculation for RGB Pixels, alter if not using RGB(single color or RGBW or something)
int MaxPixels = 170; //set in BuildCustom from the patch coordinate file
int PatchedChannels = 510; //filed in BuildCustom(), is PixelAmount * 3(not MatrixHeight * MatrixWidth) actual displayed pixels based on patch

int TargetFrameRate = 30; //user set in config, based on available bandwidth and other factors

int Xdifference = 0; //calculated using the patch coordinates
int Ydifference = 0;  //also used for external frame size

int FeedASelQuick = 2; //starting quickfeed IDs
int FeedBSelQuick = 3;

int PrevFocusA = 0;
int PrevFocusB = 0;

float SpeedA = 1;
float SpeedB = 1;

float  MixerValue = 0.5; // 0 is A, 1 is B
float  IntensityValueA =1;
float  IntensityValueB =1;

int BlendingMode = 0;
int CommunicationType = 0;
int LoadFileMethod = 0; //used in the FileLoad Switch()

boolean FeedFlag = false; //true is A, false is B
boolean FrameFirstRunA = true;
boolean FrameFirstRunB = true;
boolean AutomaticMode = false;
boolean FeedAGenerating = false;
boolean FeedBGenerating = false;
boolean GlobalDragging = false; //sliders
boolean CommunicationConnected = false;
boolean InitializedWindowSize = false;
boolean HandleSerialFlag = false;


int  PixelDataType = 0; //0 RGB, 1 RGBW, 2 GRB ..... 7 Single Color use Red, 8 use green, 9 use blue, 5 GRB, 6 BGR, 7 RBG

	//String PatchFileName = "";
	//String QuickFeedsFileName = "";
	//String AbsoluteFootagePath = "";
	//String AutomaticFileName = "";
	
String DeviceFileID = "";
String SelectedFile = "";

//Communication Variables
	//int ServerPort = 9750; //2616 is HEX
	//String ServerIP = "192.168.1.100";

boolean FramedSerialFlag = false; //also used for serial control
boolean DTRState = false; //used for polling the serial control device

int RecievedCounter = 0; //used for Serial Input control
	//boolean ConfigPreviewEnable = true;
	//boolean ConfigDisplayThumbnails = false;
	//int ConfigCOM1 = 0; //user selected COM Port, set in config.ini
	//int MIDIControlPort = 0;  //user selected set in config.ini
	//int SerialControlPort = 0;  //user selected set in config.ini
int TransmissionStatusCounter = 0; //used for indicating transmission, green circle bottom of the screen
	//int TransmissionBaud = 500000; //user selected set in config.ini
int HoldMiliSec = 0;  //
int inByte = -1;    // Incoming serial data

//GUI Variables
boolean UpdateWorkspace = true;
boolean LockIntensity = false;
boolean DisplayPreview = false;


int FeedAQuickPage = 0; //4 pages, 0 - 3
int FeedBQuickPage = 0;

int OpenMenuID = 0;
String portName = "None";

int PrevHeight = height;  //used to monitor if the grid window's size is changed
int PrevWidth = width;
float SF = 1; //scale factor

int SelectedSlider = 0;
int SelectedSliderSet = 0;

int DisplayThumbnail = 0;



int MIDIchan = 0;
int MIDInumber = 0; 
int MIDIvalue = 0;
boolean HandleMIDICC = false;
boolean HandleMIDINO = false;

//used throughout for many functions
color myColor = color(0);
float myRed  = 0;
float myGreen = 0;//moving variables out of loop should make it faster
float myBlue = 0;

//AUTOmatic mode variables
int DrawCount = 0;
int BlendSpeed = 1; //for auto-fade and automatic mode
boolean FeedID =  false;
int AutoLines = 1; //starts at 1, as line 0 is comments
boolean AutoSequenceRunning = false;
boolean TransitionFlag = false; //Auto-Fade & Automatic Variable
boolean ReverseFlag = false; //play .dat files in reverse

//Generation Vars
//system
int FrameMethod = 0; //discontinue these!!!!!!!!!!!!!!!!
int FrameMethodA = 0;
int FrameMethodB = 0;

int StepThroughVar = 0; //stays global

//Generation system variables
int HoldGenSlider1 = 0;
int HoldGenSlider2 = 0;
int HoldGenSlider3 = 0;
int HoldGenSlider4 = 0;

String[] GenSliderLabelsA = {
  "A", "B", "C", "D"
};
String[] GenSliderLabelsB = {
  "A", "B", "C", "D"
};
//String[] TempLabelArray = {"Not Used","Not Used","Not Used","Not Used"};
//END

PImage MixerHandle, OffsetMenuBG, IntensityBG, TestBars, MainMixerBG;
PGraphics GraphicsBufferA, GraphicsBufferB, buf, PreviewBuffer;

PGraphics[] CachedThumbnails = new PGraphics[60];

DCapture captureDSObj; //capture device object for DShow Input, MAC and LINUX comment out

AudioInput AudioIn; //used for LineIn
Minim minim; //dunno

Client TCPclient; // Declare a client for Processing Net library
UdpClient UDPclient; //declare UDP for netP5 library, TCP IP will not work right

Serial myPort;      // The serial port

MidiBus myBus; // The MidiBus

Movie myMovieA; //Feed A movie Object
Movie myMovieB;  //Feed B movie Object
Movie m;  //temp movie object for functions, don't really need to declare it here but seems to work better
Movie PreviewMovie; //Temp movie object for generating Previews and such

Button[] FeedAButtons;
ButtonSq[] QuickAButtons;

Button[] FeedBButtons;
ButtonSq[] QuickBButtons;

Button[] CommonButtons;
Button[] BlendButtons;
Button[] OffsetButtons;

Button[] MainMenuButtons;
Button[] MIDIMenuButtons;

SliderBar[] CommonSliders;
SliderBar[] GenerationSliders;

SliderBar MainMixerSlider;
SliderBar IntensitySliderA;
SliderBar IntensitySliderB;

SliderBar SliderPointer;

PixelDisplay[] FeedAPixels;
PixelDisplay[] FeedBPixels;
PixelDisplay[] MixedPixels;

QuickFeed[] QuickFeeds;

GUIMenu[] GUIMenus;

MatrixObj Matrix;

//generated objects
ggObjText[] genGraphicText;
ggObjImage[] genGraphicImage;
ggObjDATFile[] genGraphicDATFile;
ggObjSpinningSquare[] genGraphicSpinningSquare;
ggObjStarField[] genGraphicStarField;
ggObjRipples[] genGraphicRipples;
ggObjMetaBalls[] genGraphicMetaBalls;
ggObjPlasma[] genGraphicPlasma;

ggObjSpiral[] genGraphicSpiral;
ggObjFallingBlocks[] genGraphicFallingBlocks;
ggObjTestStepThrough[] genGraphicTestStepThrough;
 
ggObjAudioCircles[] genGraphicAudioCircles;
ggObjAudioSine[] genGraphicAudioSine;
ggObjAudioBars[] genGraphicAudioBars;
 
 
//==================== END INITIALIZATION ============================


void setup()
{
  if (InitializedWindowSize == false) 
  {
    // size(800, 600); //default
    LoadConfigFile(); //load user config.ini   
    textFont( loadFont("Gisha-Bold-32.vlw"), 24);
    ImageIcon titlebaricon = new ImageIcon(loadBytes("favicon.gif"));
    frame.setIconImage(titlebaricon.getImage());
    surface.setTitle("NLED Matrix - v2b - www.NLEDShop.com/nledmatrix - Northern Lights Electronic Design, LLC");
    
    surface.setResizable(true);
	
	
   minim = new Minim(this);
  // use the getLineIn method of the Minim object to get an AudioInput
  AudioIn = minim.getLineIn(Minim.MONO, 512);
  AudioIn.enableMonitoring();
  // uncomment this line to *hear* what is being monitored, in addition to seeing it
  AudioIn.mute();
  
  }

  background(cBackgroundColor);
  frameRate(TargetFrameRate); //draw monitors a feedback loop of sorts to maintain the Target Frame rate

  PrevHeight = height;  //used to monitor if the grid window's size is changed so objects can be redeclared
  PrevWidth = width;   

  GraphicsBufferA = createGraphics(MatrixWidth, MatrixHeight);
  GraphicsBufferB  = createGraphics(MatrixWidth, MatrixHeight);
  PreviewBuffer = createGraphics(MatrixWidth, MatrixHeight);

  PreviewBuffer.beginDraw(); //need to intialize this right away to prevent an error
  PreviewBuffer.rect(0, 0, 20, 20);
  PreviewBuffer.endDraw();

  //MaxChannels = (MatrixWidth * MatrixHeight) * 3;

  println("Setup() GUI ScaleFactor="+SF);

  //=========================== Init Objects =====================================
  GUIMenus = new GUIMenu[6];
  GUIMenus[0]= new GUIMenu("NoMenu", 0, 0, 800, 600, 0, false);    
  GUIMenus[1]= new GUIMenu("Offset Menu", 300, 300, 200, 200, 1, false);  
  GUIMenus[2]= new GUIMenu("Main Menu", 150, 250, 500, 200, 2, false);   
  GUIMenus[3]= new GUIMenu("ConfigureMenu", 150, 250, 500, 200, 3, false);  
  GUIMenus[4]= new GUIMenu("GenMenuA", 280, 400, 240, 100, 4, false);   
  GUIMenus[5]= new GUIMenu("GenMenuB", 280, 400, 240, 100, 5, false);  

  BlendButtons = new Button[7];
  BlendButtons[0] = new Button("Blend", 365, 340, 70, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, true, false);
  BlendButtons[1] = new Button("Multiply A B", 280, 370, 90, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, false, false);
  BlendButtons[2] = new Button("Multiply B A", 430, 370, 90, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, false, false);      
  BlendButtons[3] = new Button("Minim.", 280, 400, 70, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, false, false);
  BlendButtons[4] = new Button("Maxim.", 450, 400, 70, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, false, false);
  BlendButtons[5] = new Button("Add A to B", 280, 340, 80, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, false, false);
  BlendButtons[6] = new Button("Add B to A", 440, 340, 80, 25, ColButtonsBG, ColButtonsHL, cBlendButtStroke, true, false, false);

  CommonButtons = new Button[11];
  CommonButtons[0] = new Button("A", 280, 310, 40, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  CommonButtons[1] = new Button("B", 480, 310, 40, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  CommonButtons[2] = new Button("Auto Fade", 355, 310, 90, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  CommonButtons[3] = new Button("Automatic", 360, 400, 80, 25, ColButtonsBG, cAutomaticHL, cAutomaticStroke, true, false, false);
  CommonButtons[4] = new Button("Lock", 485, 450, 40, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);

  CommonButtons[5] = new Button("Save", 270, 545, 50, 25, ColButtonsBG, ColButtonsHL, cMenuButtonsStroke, false, false, false);
  CommonButtons[6] = new Button("Load", 330, 545, 50, 25, ColButtonsBG, ColButtonsHL, cMenuButtonsStroke, false, false, false);
  CommonButtons[7] = new Button("Menu", 475, 545, 60, 25, ColButtonsBG, ColButtonsHL, cMenuButtonsStroke, true, false, false);
  CommonButtons[8] = new Button("Configure", 390, 545, 75, 25, ColButtonsBG, ColButtonsHL, cMenuButtonsStroke, true, false, false);

  CommonButtons[9] = new Button("Gen. Options A", 270, 510, 120, 25, ColButtonsBG, ColButtonsHL, cMenuButtonsStroke, true, false, false);
  CommonButtons[10] = new Button("Gen. Options B", 415, 510, 120, 25, ColButtonsBG, ColButtonsHL, cMenuButtonsStroke, true, false, false);

  OffsetButtons = new Button[5];
  OffsetButtons[0] = new Button("Up", (GUIMenus[1].xpos+75), (GUIMenus[1].ypos+15), 55, 55, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  OffsetButtons[1] = new Button("Down", (GUIMenus[1].xpos+75), (GUIMenus[1].ypos+130), 55, 55, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  OffsetButtons[2] = new Button("Right", (GUIMenus[1].xpos+130), (GUIMenus[1].ypos+75), 55, 55, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  OffsetButtons[3] = new Button("Left", (GUIMenus[1].xpos+15), (GUIMenus[1].ypos+75), 55, 55, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  OffsetButtons[4] = new Button("Reset", (GUIMenus[1].xpos+75), (GUIMenus[1].ypos+75), 50, 50, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);

  FeedAButtons = new Button[14];
  FeedAButtons[0] = new Button("Play", 10, 250, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[1] = new Button("Pause", 70, 250, 60, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[2] = new Button("Stop", 140, 250, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[3] = new Button("Load", 200, 250, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  FeedAButtons[4] = new Button("Reset", 10, 285, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  FeedAButtons[5] = new Button("Invert", 10, 405, 55, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[6] = new Button("Tint", 10, 370, 55, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[7] = new Button("Color Max", 75, 370, 80, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[8] = new Button("Offset", 10, 320, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedAButtons[9] = new Button("Color Scale", 165, 370, 90, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
  FeedAButtons[10] = new Button("Color Min", 75, 405, 80, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
  FeedAButtons[11] = new Button("Contrast", 165, 405, 90, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
  FeedAButtons[12] = new Button("Pg Up", 10, 445, 50, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  FeedAButtons[13] = new Button("Pg Dwn", 65, 445, 50, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);

  QuickAButtons = new ButtonSq[60];
  int q = 0;
  for (int i=0; i != 15; i++)
  {
    if (i < 5)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+(i*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (i > 4 && i < 10)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((i-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((i-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  } 
  q = 0;
  for (int i=15; i != 30; i++)
  {
    if (q < 5)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+(q*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (q > 4 && q < 10)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((q-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((q-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
    q++;
  } 
  q = 0;
  for (int i=30; i != 45; i++)
  {
    if (q < 5)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+(q*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (q > 4 && q < 10)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((q-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((q-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
    q++;
  } 
  q = 0;
  for (int i=45; i != 60; i++)
  {
    if (q < 5)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+(q*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (q > 4 && q < 10)  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((q-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickAButtons[i] = new ButtonSq("Q"+(i+1), (10+((q-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
    q++;
  }    

  //======================================================================

  QuickBButtons = new ButtonSq[60];
  q = 0;
  for (int i=0; i != 15; i++)
  {
    if (i < 5)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+(i*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (i > 4 && i < 10)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((i-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((i-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  } 
  q = 0;
  for (int i=15; i != 30; i++)
  {
    if (q < 5)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+(q*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (q > 4 && q < 10)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((q-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((q-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
    q++;
  } 
  q = 0;
  for (int i=30; i != 45; i++)
  {
    if (q < 5)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+(q*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (q > 4 && q < 10)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((q-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((q-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
    q++;
  } 
  q = 0;
  for (int i=45; i != 60; i++)
  {
    if (q < 5)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+(q*50)), 475, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else if (q > 4 && q < 10)  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((q-5)*50)), 515, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
    else  QuickBButtons[i] = new ButtonSq("Q"+(i+1), (550+((q-10)*50)), 555, 45, 35, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
    q++;
  }   

  //========================================

  QuickAButtons[FeedASelQuick].Selected = true;
  QuickBButtons[FeedBSelQuick].Selected = true;

  FeedBButtons = new Button[14];
  FeedBButtons[0] = new Button("Play", 550, 250, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[1] = new Button("Pause", 610, 250, 60, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[2] = new Button("Stop", 680, 250, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[3] = new Button("Load", 740, 250, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  FeedBButtons[4] = new Button("Reset", 550, 285, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  FeedBButtons[5] = new Button("Invert", 550, 405, 55, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[6] = new Button("Tint", 550, 370, 55, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[7] = new Button("Color Max", 615, 370, 80, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[8] = new Button("Offset", 550, 320, 50, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  FeedBButtons[9] = new Button("Color Scale", 705, 370, 90, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
  FeedBButtons[10] = new Button("Color Min", 615, 405, 80, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
  FeedBButtons[11] = new Button("Contrast", 705, 405, 90, 25, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);  
  FeedBButtons[12] = new Button("Pg Up", 550, 445, 50, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  FeedBButtons[13] = new Button("Pg Dwn", 605, 445, 50, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);

  //===================== START MENU OBJECTS =====================

  int tXpos = GUIMenus[2].xpos;//main menu location
  int tYpos = GUIMenus[2].ypos;

  MainMenuButtons = new Button[13];
  MainMenuButtons[0] = new Button("Save Device Config", 100+tXpos, 10+tYpos, 120, 20, ColButtonsBG, ColButtonsHL, ColButtonsHL, false, false, false);
  MainMenuButtons[1] = new Button("Load Device Config", 230+tXpos, 10+tYpos, 120, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  MainMenuButtons[2] = new Button("Enable Serial Control", 10+tXpos, 40+tYpos, 130, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[3] = new Button("Enable DShow Feed", 150+tXpos, 40+tYpos, 120, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[4] = new Button("Large Previews", 290+tXpos, 40+tYpos, 90, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[5] = new Button("Enable MIDI", 390+tXpos, 40+tYpos, 90, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);

  MainMenuButtons[6] = new Button("None", 110+tXpos, 145+tYpos, 40, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[7] = new Button("Serial", 160+tXpos, 145+tYpos, 40, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[8] = new Button("TCP", 210+tXpos, 145+tYpos, 40, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[9] = new Button("UDP", 260+tXpos, 145+tYpos, 40, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, true, false, false);
  MainMenuButtons[10] = new Button("EULA", 450+tXpos, 10+tYpos, 40, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  MainMenuButtons[11] = new Button("Rebuild Thumbnails", 300+tXpos, 170+tYpos, 120, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);
  MainMenuButtons[12] = new Button("Website", 360+tXpos, 10+tYpos, 60, 20, ColButtonsBG, ColButtonsHL, ColButtonsOut, false, false, false);

  //set Intial button states
  MainMenuButtons[2].Selected = Matrix.softwareSerialConEnable;
  MainMenuButtons[3].Selected = Matrix.softwareEnableDSVideoFeed;
  MainMenuButtons[4].Selected = Matrix.softwareLrgPreviewEnable;
  MainMenuButtons[5].Selected = Matrix.softwareMIDIenable;
  MainMenuButtons[6+Matrix.matrixCommMethod].Selected = true;

  //============== START SLIDERS ========================================

  MainMixerSlider = new SliderBar(290, 260, 220, 40, 50, 0, 100, color(255), color(0), color(200, 200, 200), color(0), true, false, 0); //mixer
  IntensitySliderA =  new SliderBar(290, 440, 180, 20, 100, 0, 100, color(255), color(0), color(200, 200, 200), color(0), false, false, 1); //intesity
  IntensitySliderB =  new SliderBar(290, 470, 180, 20, 100, 0, 100, color(255), color(0), color(200, 200, 200), color(0), false, false, 2); //intesity

  CommonSliders = new SliderBar[10];
  CommonSliders[0] = new SliderBar(130, 305, 120, 16, 255, 0, 255, color(255), color(0), color(255, 0, 0), cSliderStrokeColor, true, false, 3); //Feed A Red
  CommonSliders[1] = new SliderBar(130, 325, 120, 16, 255, 0, 255, color(255), color(0), color(0, 255, 0), cSliderStrokeColor, true, false, 3); //Feed A Green
  CommonSliders[2] = new SliderBar(130, 345, 120, 16, 255, 0, 255, color(255), color(0), color(0, 0, 255), cSliderStrokeColor, true, false, 3); //Feed A Blue
  //Feed B Color
  CommonSliders[3] = new SliderBar(670, 305, 120, 16, 255, 0, 255, color(255), color(0), color(255, 0, 0), cSliderStrokeColor, true, false, 3); //Feed A Red
  CommonSliders[4] = new SliderBar(670, 325, 120, 16, 255, 0, 255, color(255), color(0), color(0, 255, 0), cSliderStrokeColor, true, false, 3); //Feed A Green
  CommonSliders[5] = new SliderBar(670, 345, 120, 16, 255, 0, 255, color(255), color(0), color(0, 0, 255), cSliderStrokeColor, true, false, 3); //Feed A Blue
  //speed
  CommonSliders[6] = new SliderBar(140, 285, 110, 14, cDefaultSpeed, 0, 200, cSliderBGColor, cSliderHndlColor, cSliderHLColor, color(0), false, false, 3); //Feed A Speed
  CommonSliders[7] = new SliderBar(680, 285, 110, 14, cDefaultSpeed, 0, 200, cSliderBGColor, cSliderHndlColor, cSliderHLColor, color(0), false, false, 3); //Feed B Speed
  //contrast
  CommonSliders[8] = new SliderBar(135, 450, 120, 14, 0, 0, 255, cSliderBGColor, cSliderHndlColor, cSliderHLColor, cSliderStrokeColor, false, false, 3); //Feed A Contrast
  CommonSliders[9] = new SliderBar(670, 450, 120, 14, 0, 0, 255, cSliderBGColor, cSliderHndlColor, cSliderHLColor, cSliderStrokeColor, false, false, 3); //Feed B Contrast

  GenerationSliders = new SliderBar[4];
  GenerationSliders[0] = new SliderBar(290, 430, 90, 16, 1, 0, 255, cSliderBGColor, cSliderHndlColor, cSliderHLColor, cSliderStrokeColor, false, false, 4);
  GenerationSliders[1] = new SliderBar(420, 430, 90, 16, 1, 0, 255, cSliderBGColor, cSliderHndlColor, cSliderHLColor, cSliderStrokeColor, false, false, 4);
  GenerationSliders[2] = new SliderBar(290, 470, 90, 16, 1, 0, 255, cSliderBGColor, cSliderHndlColor, cSliderHLColor, cSliderStrokeColor, false, false, 4);
  GenerationSliders[3] = new SliderBar(420, 470, 90, 16, 1, 0, 255, cSliderBGColor, cSliderHndlColor, cSliderHLColor, cSliderStrokeColor, false, false, 4);

  SliderPointer = new SliderBar(0, 0, 0, 0, 0, 0, 0, color(0), color(0), color(0), color(0), true, false, 255); //Pointer for Keyboard control of sliders

  //========================= END SLIDERS ===================================================

  FeedAPixels= new PixelDisplay[(MatrixWidth * MatrixHeight)];
  FeedBPixels= new PixelDisplay[(MatrixWidth * MatrixHeight)];
  MixedPixels= new PixelDisplay[(MatrixWidth * MatrixHeight)];   //were all 340

   
	if (InitializedWindowSize == false) 
	{
	//load generation objects 
	genGraphicText = new ggObjText[4];
	genGraphicImage = new ggObjImage[4]; 
	genGraphicDATFile = new ggObjDATFile[4];
	genGraphicSpinningSquare = new ggObjSpinningSquare[4];
	genGraphicStarField = new ggObjStarField[4];
	genGraphicRipples = new ggObjRipples[4];
	genGraphicMetaBalls = new ggObjMetaBalls[4];
	genGraphicPlasma = new ggObjPlasma[4];
	genGraphicSpiral = new ggObjSpiral[4];
	genGraphicFallingBlocks = new ggObjFallingBlocks[4];
	genGraphicTestStepThrough = new ggObjTestStepThrough[4]; 

	genGraphicAudioCircles = new ggObjAudioCircles[4]; 
	genGraphicAudioSine = new ggObjAudioSine[4]; 
	genGraphicAudioBars = new ggObjAudioBars[4]; 
	//end 
	}
	 
  BuildCustom(); //build Pixel Displays and load coordinate file into arrays

  QuickFeeds = new QuickFeed[60];
  LoadQuickFeedsFile(); //Loads QuickFeeds

  MixerHandle = loadImage("mixerhandle.png");  
  OffsetMenuBG = loadImage("offsetmenu.png");
  IntensityBG = loadImage("intensitybg.png");
  TestBars = loadImage("testbars.jpg");
  MainMixerBG = loadImage("mainmixerbg.png");

  //======================================================================

  OpenConnection(); //opens communcation based on Matrix.matrixCommMethod variable

  //======================================================================

  if (Matrix.softwareSerialConEnable == true)
  {
    try {
      println(Serial.list());
      portName = Serial.list()[Matrix.softwareSerialConPort];
      myPort = new Serial(this, portName, Matrix.softwareSerialConBaud);    
      println("Connected to: "+portName);
    }
    catch(Exception e)
    {
      println("Serial Control COM Port Error");
    }
  }

  //=============================================================

  if (Matrix.softwareMIDIenable == true && myBus == null)
  {
    InitializeMIDI(true);
    InitializeMIDI(false);
  }

  if (InitializedWindowSize == false)
  {
    myMovieA = new Movie(this, "");
    myMovieB = new Movie(this, "");
    myMovieA.frameRate(30);
    myMovieB.frameRate(30);
    myMovieA.loop(); 
    myMovieB.loop();     
    LoadQuickFeed(myMovieA, QuickFeeds[FeedASelQuick]);
    LoadQuickFeed(myMovieB, QuickFeeds[FeedBSelQuick]);
  }

  SetControlStateFunc(myMovieA, FeedAButtons, QuickFeeds[FeedASelQuick].ControlState);
  SetControlStateFunc(myMovieB, FeedBButtons, QuickFeeds[FeedBSelQuick].ControlState);  

  //ADDED =========================================

  if(Matrix.softwareAuroraCMD)
  {
  try {
  myPort.write("N"); 
  myPort.write("L");  
  myPort.write("E");   
  myPort.write("D");  
  myPort.write("1");   
  myPort.write("1");      
  delay(5);
  myPort.write("n"); 
  myPort.write("l");  
  myPort.write("e");   
  myPort.write("d");  
  myPort.write("9");   
  myPort.write("9");
  delay(5);  
  myPort.write(60); 
  myPort.write(1);   
  myPort.write(0); 
  println("PatchedChannels: "+PatchedChannels);
  myPort.write((PatchedChannels >> 8) & 0xFF); 
  myPort.write(PatchedChannels & 0xFF);     
  }
  catch(Exception e) { println("No COM Port open to send NLED Aurora Live Control CMD to"); }
}

  /*
  myPort.write("q"); 
  myPort.write("Q");  
  myPort.write("z");       
  delay(5);
  myPort.write("G"); 
  delay(5);  
  myPort.write(60); 
  myPort.write(1);   
  myPort.write(0); 
*/

//host sends to device -> q -> Q -> z -> device sends N -> host responds with G -> device sends n -> Host sends -> Command -> data1 -> data2


  //END ADDED =========================================


  InitializedWindowSize = true;
}//end setup()

//=========================================================

void SendData()
{
  //To send lots of data efficiently it must be sent in a byte array, otherwise each byte will few mS space between them
  //  so it would be real slow, sending as an array[] sends it fast as possible, in large packets.

  //============== PREPARE DATA =====================
  //Color Conversions for nonstandard RGB and single color usage is done here

  int x = 0; //used in switch

  switch(Matrix.matrixPixelDataType)
  {
  case 0: //RGB
      for (int i=0; i != PatchedChannels; i++)  TransmissionArray[i] = byte(MixedBuffer[i]); //transfer Mixed buffer into a byte array for faster transmission  
    break;
  case 1: //RGBW
    //NOT IMPLEMENTED
    break;  
  case 2: //GRB output, WS2812B, among others
    for (int i=0; i != PatchedChannels; i=i+3) 
    {
      TransmissionArray[i] = byte(MixedBuffer[x+1]); //transfer Mixed buffer into a byte array for faster transmission
      TransmissionArray[i+1] = byte(MixedBuffer[x]); //transfer Mixed buffer into a byte array for faster transmission
      TransmissionArray[i+2] = byte(MixedBuffer[x+2]); //transfer Mixed buffer into a byte array for faster transmission    
      x+=3;
    }
    break;  
  case 3: //BRG output, 

    for (int i=0; i != PatchedChannels; i=i+3) 
    {
      TransmissionArray[i] = byte(MixedBuffer[x+2]); //transfer Mixed buffer into a byte array for faster transmission
      TransmissionArray[i+1] = byte(MixedBuffer[x]); //transfer Mixed buffer into a byte array for faster transmission
      TransmissionArray[i+2] = byte(MixedBuffer[x+1]); //transfer Mixed buffer into a byte array for faster transmission    
      x+=3;
    }

    break;  

  case 4: //GBR output

    break;  

  case 5: //RBG output

    break; 

  case 6: //BGR output, 

    break;  

  case 7: //Single Use Red Channel
    for (int i=0; i != MaxPixels; i++) 
    {
      TransmissionArray[i] = byte(MixedBuffer[x]); //pulls just the red out
      x+=3;
    } 
    break;  
  case 8: //Single Use Green Channel
    x = 1;
    for (int i=0; i != MaxPixels; i++) 
    {
      TransmissionArray[i] = byte(MixedBuffer[x]); //pulls just the red out
      x+=3;
    }
    break;  
  case 9: //Single Use Blue Channel
    x = 2;
    for (int i=0; i != MaxPixels; i++) 
    {
      TransmissionArray[i] = byte(MixedBuffer[x]); //pulls just the red out
      x+=3;
    }  
    break;
  }
  //============== Send Out =============================
  try { 
    if (Matrix.matrixCommMethod == 1)  myPort.write(TransmissionArray);  //sends as an array, much much faster than other methods, only works with bytes()
    else if (Matrix.matrixCommMethod == 2)  UDPclient.send(TransmissionArray); //netP5 UDP Connection
    else if (Matrix.matrixCommMethod == 3)  TCPclient.write(TransmissionArray); //Processing Net TCP Connection
  }
  catch(Exception e)
  {
    //if the communcation type errors, turn off and continue
    Matrix.matrixCommMethod = 0;
    CommunicationConnected = false;
    println("Cannot Send, Closing Communication Port");
  }

  TransmissionStatusCounter++;
}//end func


void serialEvent(Serial myPort)
{
  //currently only reads the Serial Control data, no other use
  inByte = myPort.read();
  // println(int(inByte));
  int tempTime = (millis() - HoldMiliSec);
  println("Received: "+inByte+"/"+char(inByte)+" Time: "+tempTime);
  HoldMiliSec=millis();  

  if (tempTime > 20 && tempTime < 110)
  {
    RecievedCounter = 0; 
    FramedSerialFlag = true;
  }

  if (Matrix.softwareSerialConEnable == true && FramedSerialFlag == true)
  {
    //if receiving commands over serial and there has been a 10mSec Space since last byte, this is the first of the packet
    ReceivedSerialControl[RecievedCounter] =  inByte;
    RecievedCounter++;

    if (RecievedCounter == 11)
    {  
      FramedSerialFlag = false;  //clear flag
      //  println(ReceivedSerialControl); 
      HandleSerialFlag = true;
    }//end if
  } //end if &&
}//end func

void movieEvent(Movie m) {
  try {
    m.read();
  }
  catch(Exception e)
  {
    println("MovieEvent() Error, can not read()");
  }
} //end func

//=============================================================

void OpenConnection()
{
  //Matrix.matrixCommMethod = 1; //override if necassary

  if (CommunicationConnected == false) //checks if already connected
  {  
    switch(Matrix.matrixCommMethod) //switches to select the communcation type
    {
    case 0: //no communication
      println("No Communicaiton Opened");
      portName = "None";
      break;
    case 1: //serial on single COM port
      //Tries to connect to the serial port specified in the config.ini
      if (myPort != null) myPort.stop();

      try
      {
        println(Serial.list());
        portName = Serial.list()[Matrix.matrixCOM1];
        myPort = new Serial(this, portName, Matrix.matrixTXBaud);   // 1000000);
        CommunicationConnected = true;  
        println("Connected to: "+Serial.list()[Matrix.matrixCOM1]);
      }
      catch(Exception e)
      {
        println("Cannot Open Specified Port, Please Check config.ini");
      }
      break;
    case 2: //UDP
      try {
        UDPclient = new UdpClient(Matrix.matrixServerIP, Matrix.matrixServerPort);
        CommunicationConnected = true;  
        portName = "UDP";
        println("Attempting UDP Connecton to "+Matrix.matrixServerIP+" on port "+Matrix.matrixServerPort);
      }
      catch(Exception e)
      {
        println("Could Not Start UDP Transmission");
      }
      break;
    case 3: //TCP/IP
      try {
        //   TCPclient = new TcpClient(this, Matrix.matrixServerIP, Matrix.matrixServerPort, 3);

        TCPclient = new Client(this, Matrix.matrixServerIP, Matrix.matrixServerPort);
        CommunicationConnected = true;  
        portName = "TCP/IP";
        println("Connected to TCP Server at "+Matrix.matrixServerIP+" on port "+Matrix.matrixServerPort);
      }
      catch(Exception e)
      {
        println("Could Not Connect To TCP/IP Server");
      }
      break;
    }//end switch
  }//end Connected If()
}